@@ -7,6 +7,8 @@ from django.db import models
 from django.conf import settings
 from django.db.models import Count,Sum,Q
 
+from picklefield.fields import PickledObjectField
+
 # Create your models here.
 
 
@@ -514,6 +516,7 @@ class WordToken(models.Model):
     FollSemPred = models.FloatField(blank=True,null=True)
     NumFormants = models.DecimalField('Number of formants',max_digits=4,decimal_places=1,blank=True,null=True)
     Ceiling = models.IntegerField(blank=True,null=True)
+    Output = PickledObjectField()
 
     objects = WTManager()
 
@@ -919,98 +922,172 @@ class WordToken(models.Model):
         return True
 
     def getAnalysisLines(self,form):
-        beg,end,vow,prevSound,follSound = self.getAcousticInfo()
-        outline = {'Word':self.WordType.Label}
-        outline['Vowel'] = self.WordType.getStressVowel()
-        outline['PrevCons'] = prevSound
-        outline['FollCons'] = follSound
-        outline['Token'] = self.pk
+        if self.Output is not None:
+            outline = copy(self.Output)
+        else:
+            outline = {}
+        wanted_fields =[]
+        if outline == {}:
+            outline['Word'] = self.WordType.Label
+            outline['Token'] = self.pk
+            beg,end,vow,prevSound,follSound = self.getAcousticInfo()
+            outline['Vowel'] = self.WordType.getStressVowel()
+            outline['PrevCons'] = prevSound
+            outline['FollCons'] = follSound
+            outline['Speaker'] = str(self.Dialog.Speaker)
+        wanted_fields.extend(['Word','Token','Vowel','PrevCons','FollCons','Speaker'])
         if form['segmentalDurations']:
-            outline['VowDur'] = end-beg
-            outline['OtherDur'] = self.getDuration()- outline['VowDur']
+            if 'VowDur' not in outline:
+                outline['VowDur'] = end-beg
+                outline['OtherDur'] = self.getDuration()- outline['VowDur']
+            wanted_fields.extend(['VowDur','OtherDur'])
         if form['speakingRates']:
-            outline['PrevSpeakRate'] = self.getPreviousSpeakingRate()
-            outline['FollSpeakRate'] = self.getFollowingSpeakingRate()
+            if 'PrevSpeakRate' not in outline:
+                outline['PrevSpeakRate'] = self.getPreviousSpeakingRate()
+                outline['FollSpeakRate'] = self.getFollowingSpeakingRate()
+                outline['AvgSpeakRate'] = self.Dialog.Speaker.getAvgSpeakingRate()
+            wanted_fields.extend(['PrevSpeakRate','FollSpeakRate','AvgSpeakRate'])
         if form['contextProbs']:
-            outline['PrevCondProb'] = self.getPreviousCondProb()
-            outline['FollCondProb'] = self.getFollowingCondProb()
+            if 'PrevCondProb' not in outline:
+                outline['PrevCondProb'] = self.getPreviousCondProb()
+                outline['FollCondProb'] = self.getFollowingCondProb()
+            wanted_fields.extend(['PrevCondProb','FollCondProb'])
         if form['repetitions']:
-            outline['Repetitions'] = self.getRepetitions()
-            outline['wasRepeatedRecently'] = self.getRecentRepetition()
+            if 'Repetitions' not in outline:
+                outline['Repetitions'] = self.getRepetitions()
+                outline['wasRepeatedRecently'] = self.getRecentRepetition()
+            wanted_fields.extend(['Repetitions','wasRepeatedRecently'])
         if form['wasRepeated']:
-            if 'Repetitions' in outline:
+            if 'Repetitions' not in outline:
+                outline['Repetitions'] = self.getRepetitions()
+            if 'wasRepeated' not in outline:
                 if outline['Repetitions'] != 0:
                     outline['wasRepeated'] = 'True'
                 else:
                     outline['wasRepeated'] = 'False'
-            else:
-                if self.getRepetitions() != 0:
-                    outline['wasRepeated'] = 'True'
-                else:
-                    outline['wasRepeated'] = 'False'
+            wanted_fields.extend(['wasRepeated'])
+
+
         if 'B' in form['category']:
-            outline['BuckeyeCategory'] = self.Category.Label
+            if 'BuckeyeCategory' not in outline:
+                outline['BuckeyeCategory'] = self.Category.Label
+            wanted_fields.extend(['BuckeyeCategory'])
         if 'C' in form['category']:
-            outline['CelexCategory'] = lookupCat(self.WordType.Label)
+            if 'CelexCategory' not in outline:
+                outline['CelexCategory'] = lookupCat(self.WordType.Label)
+            wanted_fields.extend(['CelexCategory'])
+
         if form['wordDuration']:
-            outline['WordDuration'] = self.getDuration()
-            outline['BaselineDur'] = self.WordType.getBaseDuration()
+            if 'WordDuration' not in outline:
+                outline['WordDuration'] = self.getDuration()
+                outline['BaselineDur'] = self.WordType.getBaseDuration()
+            wanted_fields.extend(['WordDuration','BaselineDur'])
+
         if 'B' in form['frequency']:
-            outline['BuckeyeFrequency'] = self.WordType.getFreq()
+            if 'BuckeyeFrequency' not in outline:
+                outline['BuckeyeFrequency'] = self.WordType.getFreq()
+            wanted_fields.extend(['BuckeyeFrequency'])
         if 'C' in form['frequency']:
-            outline['CelexFrequency'] = lookupFreq(self.WordType.Label)
+            if 'CelexFrequency' not in outline:
+                outline['CelexFrequency'] = lookupFreq(self.WordType.Label)
+            wanted_fields.extend(['CelexFrequency'])
+
         if form['gender']:
-            outline['SpeakerGender'] = self.Dialog.Speaker.Gender
+            if 'SpeakerGender' not in outline:
+                outline['SpeakerGender'] = self.Dialog.Speaker.Gender
+            wanted_fields.extend(['SpeakerGender'])
         if form['age']:
-            outline['SpeakerAge'] = self.Dialog.Speaker.Age
-        outline['Speaker'] = str(self.Dialog.Speaker)
+            if 'SpeakerAge' not in outline:
+                outline['SpeakerAge'] = self.Dialog.Speaker.Age
+            wanted_fields.extend(['SpeakerAge'])
+
         if form['orthoLength']:
-            outline['OrthoLength'] = len(self.WordType.Label)
+            if 'OrthoLength' not in outline:
+                outline['OrthoLength'] = len(self.WordType.Label)
+            wanted_fields.extend(['OrthoLength'])
         if form['phonoLength']:
-            outline['PhonoLength'] = self.WordType.UR.count()
+            if 'PhonoLength' not in outline:
+                outline['PhonoLength'] = self.WordType.UR.count()
+            wanted_fields.extend(['PhonoLength'])
         if 'B' in form['nd']:
-            outline['BuckeyeNeighDen'],outline['BuckeyeFWND'] = self.WordType.getNDs()
+            if 'BuckeyeNeighDen' not in outline:
+                outline['BuckeyeNeighDen'],outline['BuckeyeFWND'] = self.WordType.getNDs()
+            wanted_fields.extend(['BuckeyeNeighDen','BuckeyeFWND'])
         if 'PS' in form['nd']:
-            outline['PhonoStatsNeighDen'] = getNeighCount(self.WordType.getStrUR(),no_stress=True)
+            if 'PhonoStatsNeighDen' not in outline:
+                outline['PhonoStatsNeighDen'] = getNeighCount(self.WordType.getStrUR(),no_stress=True)
+            wanted_fields.extend(['PhonoStatsNeighDen'])
+
         if 'VB' in form['phonotactic']:
-            outline['BuckeyeSPhoneProb'],outline['BuckeyeBiPhoneProb'] = self.WordType.getPhonoProb()
+            if 'BuckeyeSPhoneProb' not in outline:
+                outline['BuckeyeSPhoneProb'],outline['BuckeyeBiPhoneProb'] = self.WordType.getPhonoProb()
+            wanted_fields.extend(['BuckeyeSPhoneProb','BuckeyeBiPhoneProb'])
         if 'B' in form['phonotactic']:
-            outline['PhonoStatsBlickPhono'] = getPhonotacticProb(self.WordType.getStrUR(),use_blick=True,no_stress=True)
+            if 'PhonoStatsBlickPhono' not in outline:
+                outline['PhonoStatsBlickPhono'] = getPhonotacticProb(self.WordType.getStrUR(),use_blick=True,no_stress=True)
+            wanted_fields.extend(['PhonoStatsBlickPhono'])
         if 'VP' in form['phonotactic']:
-            outline['PhonoStatsSPhoneProb'],outline['PhonoStatsBiPhoneProb'] = getPhonotacticProb(self.WordType.getStrUR(),use_blick=False,no_stress=True)
+            if 'PhonoStatsSPhoneProb' not in outline:
+                outline['PhonoStatsSPhoneProb'],outline['PhonoStatsBiPhoneProb'] = getPhonotacticProb(self.WordType.getStrUR(),use_blick=False,no_stress=True)
+            wanted_fields.extend(['PhonoStatsSPhoneProb','PhonoStatsBiPhoneProb'])
         if form['prev_sem_pred']:
-            outline['PrevSemPred'] = self.getPrevSemPred()
-            #outline['ImmedSemPred'] = self.getImmediateSemPred()
+            if 'PrevSemPred' not in outline:
+                outline['PrevSemPred'] = self.getPrevSemPred()
+                #outline['ImmedSemPred'] = self.getImmediateSemPred()
+            wanted_fields.extend(['PrevSemPred'])
         if form['foll_sem_pred']:
-            outline['FollSemPred'] = self.getFollSemPred()
+            if 'FollSemPred' not in outline:
+                outline['FollSemPred'] = self.getFollSemPred()
+            wanted_fields.extend(['FollSemPred'])
         if form['placeInDialog']:
-            outline['placeInDialog'] = self.getDialogPlace()
-        outline['AvgSpeakRate'] = self.Dialog.Speaker.getAvgSpeakingRate()
+            if 'placeInDialog' not in outline:
+                outline['placeInDialog'] = self.getDialogPlace()
+            wanted_fields.extend(['placeInDialog'])
         if form['measure'] == 'S':
-            ceiling = self.getCeiling()
-            nformants = self.getNFormants()
-            p.get_formants(self.getDialogPath(),beg,end,nformants,ceiling)
-            return out
+            if 'formants' not in outline:
+                ceiling = self.getCeiling()
+                nformants = self.getNFormants()
+                outline['formants'] = [ {'time(s)':x['time(s)'],
+                            'F1(Hz)':x['F1(Hz)'],
+                            'F2(Hz)':x['F2(Hz)'}
+                            for p.get_formants(self.getDialogPath(),beg,end,nformants,ceiling)
+                                if x['F1(Hz)'] != '--undefined--' and x['F2(Hz)'] != '--undefined--']
+            to_out = [ {'time(s)':x['time(s)'],
+                            'F1(Hz)':x['F1(Hz)'],
+                            'F2(Hz)':x['F2(Hz)'}.update({k:outline[k] for k in outline
+                                                            if k != 'formants' and k in wanted_fields})
+                            for x in outline['formants']]
         else:
-            F1 = self.getStrF1()
-            F2 = self.getStrF2()
-            if F1 is None:
-                outline['F1'] = 'NA'
-            else:
-                outline['F1'] = F1
-            if F2 is None:
-                outline['F2'] = 'NA'
-            else:
-                outline['F2'] = F2
+            if 'F1' not in outline:
+                F1 = self.getStrF1()
+                F2 = self.getStrF2()
+                if F1 is None:
+                    outline['F1'] = 'NA'
+                else:
+                    outline['F1'] = F1
+                if F2 is None:
+                    outline['F2'] = 'NA'
+                else:
+                    outline['F2'] = F2
+            wanted_fields.extend(['F1','F2'])
             if form['measure'] == 'D' and 'NA' not in [outline['F1'],outline['F2']]:
                 if form['DispersionFromAH']:
-                    center = self.Dialog.Speaker.getAHCenter()
+                    if 'AHDispersion' not in outline:
+                        center = self.Dialog.Speaker.getAHCenter()
+                        outline['AHDispersion'] = math.sqrt(math.pow(outline['F1']-center[0],2)+math.pow(outline['F2']-center[1],2))
+                    wanted_fields.extend(['AHDispersion'])
                 else:
-                    center = self.Dialog.Speaker.getCenter()
-                outline['Dispersion'] = math.sqrt(math.pow(outline['F1']-center[0],2)+math.pow(outline['F2']-center[1],2))
+                    if 'Dispersion' not in outline:
+                        center = self.Dialog.Speaker.getCenter()
+                        outline['Dispersion'] = math.sqrt(math.pow(outline['F1']-center[0],2)+math.pow(outline['F2']-center[1],2))
+                    wanted_fields.extend(['Dispersion'])
             elif form['measure'] == 'D':
                 outline['Dispersion'] = 'NA'
-        return [outline]
+        to_out = [{k:outline[k] for k in outline if k in wanted_fields}]
+        if outline.keys() != self.Output.keys():
+            self.Output = outline
+            self.save()
+        return to_out
 
 #class SynSemCase(models.Model):
     #verb = models.ForeignKey(WordToken)
