@@ -94,10 +94,7 @@ def basic(request):
         if request.method == 'POST':
             form = AnalysisForm(request.POST)
             if form.is_valid():
-                if settings.DEBUG:
-                    testAnalysis(form.cleaned_data)
-                else:
-                    doBasicAnalysis.delay(form.cleaned_data)
+                doBasicAnalysis.delay(form.cleaned_data)
                 return redirect(index)
         form = AnalysisForm()
         return render(request,'buckeyebrowser/form.html',{'form':form})
